<?php
/* SVN FILE: $Id: comments.php 307 2008-05-09 22:53:36Z bb_yujiro $ */
/**
 * Model_View Controller
 *
 * PHP versions 5
 *
 *      Hitsuji : Social Network Service <http://rakuto.net/rktSNS/>
 *      Copyright (c) 2007 Yujiro Takahashi
 *
 * Licensed under The MIT License
 * Redistributions of files must retain the above copyright notice.
 *
 * @filesource
 * @package         Hitsuji
 * @copyright       Copyright (c) 2007 Yujiro Takahashi
 * @link            http://rakuto.net/
 * @author          Yujiro Takahashi <yujiro@rakuto.net>
 * @version         $Revision: 307 $
 * @modifiedby      $LastChangedBy: bb_yujiro $
 * @lastmodified    $Date: 2008-05-10 07:53:36 +0900 (土, 10 5 2008) $
 * @license         http://opensource.org/licenses/mit-license.php The MIT License
 */

/**
 * Controllerクラス
 *
 * @category        Hitsuji : Controller
 * @package         Hitsuji
 * @copyright       Copyright (c) 2007 Yujiro Takahashi
 * @link            http://rakuto.net/rktSNS/
 * @author          Yujiro Takahashi <yujiro@rakuto.net>
 * @version         $Revision: 307 $
 * @modifiedby      $LastChangedBy: bb_yujiro $
 * @lastmodified    $Date: 2008-05-10 07:53:36 +0900 (土, 10 5 2008) $
 * @license         http://opensource.org/licenses/mit-license.php The MIT License
 */
class Controller extends Community_Controller 
{
    /**
     * 使用モデル名
     * @var array
     */
    public $models = array(
        'manip'=> array(
        ),
        'view'=> array(
            'com_comment',
        ),            
    );

    /**
     * com_comment モデルオブジェクト
     * @var object 
     */
    public $v_com_comment = null;

    /**
     * トピックID
     * @var integer
     */
    protected $ref_topic = null;

    /**
     * サマリーアクション
     * @var string
     */
    protected $summary = null;

    /**
     * サマリーリスト
     * @var array
     */
    private $_list = array();

    /**
     * 変数の初期化
     *
     * @abstract
     * @access protected
     * @return void
     */
    protected function initValue()
    {
        $this->v_com_comment->setValue('ref_community', $this->community['id']);
        if (!empty($this->ref_topic)){
            $this->v_com_comment->setValue('ref_topic', $this->ref_topic);
        }

        $this->v_com_comment->setOrder('regist_date DESC');
        $this->v_com_comment->setLimit(LIMIT_ROWS);
    }

    /**
     * 実行処理
     *
     * @access public
     * @return void
     **/
    protected function action()
    {
        $command = '';
        if (!empty($this->ref_topic)){
            $topic = RKT_DB::getRow('com_topic', $this->ref_topic);
            $this->objvew->assign('topic', $topic);
            if (!($this->visible & $topic['visibility'])){
                throw new RKT_Exception(RKT_Exception::ACCESS_DENIED);
            }
        } else {
            if (!($this->permission & PERMISSION_MEMBERS)){
                throw new RKT_Exception(RKT_Exception::ACCESS_DENIED);
            }
            $command = '<img src="'.BASE_URL.'resource/img/icons/bulletin.gif"'.
                       ' width="16" height="16" alt="&gt;"/><a href="'.HTTP_ACTION.
                       'community/forum/view/%s/">'._('View Topic').'</a><br>';
        }
        $this->objvew->assign('command', $command);
        $this->_setSummary();
    }

    /**
     * 入力条件の設定
     *
     * @access protected
     * @return void
     **/
    protected function setCondition()
    {
        $this->conditions = array (
            'ref_topic'=> array(
                'type'=>     'number',
                'option'=>   array(),
                'required'=> true,
                'pic_up'=>   'PathArgs'
            ),
            'summary'=> array(
                'type'=>     'string',
                'option'=>   array(),
                'required'=> true,
                'pic_up'=>   'PathArgs'
            ),
        );
    }

    /**
     * Summaryデータの設定
     *
     * @access private
     * @return void
     **/
    private function _setSummary()
    {
        if (empty($this->summary)){
            return ;
        }

        $result = $this->v_com_comment->getValues();
        foreach ($result as $row){
            /* トピック */
            $topic = RKT_DB::getRow('com_topic', $row['ref_topic'], 'topic');

            /* プロフィール */
            $condition = array(
                'ref_account'=> $row['ref_author']
            );
            $profile = RKT_DB::getRow('act_profile', $condition, 'nickname');

            $this->_list[] = array(
                'title'=>       $topic['topic'],
                'link'=>        HTTP_ACTION.'community/forum/view/'.$row['ref_topic'].'/',
                'modified'=>    $row['modified'],
                'date'=>        fDate($row['modified'], _('Y-m-d')),
                'creator'=>     $profile['nickname'],
                'description'=> strip_tags($row['comment']),
            );
        }
        
        if ($this->summary == 'json'){
            $this->_setJson();
        } else {
            $this->_setXml();
        }
    }

    /**
     * JSONデータの設定
     *
     * @access private
     * @return void
     **/
    private function _setJson()
    {
        $this->objvew->setBaseFile('common/frame/json.tpl');
        $this->objvew->setMimeType('application/json');
        
        $this->objvew->assign('data', json_encode($this->_list));
    }

    /**
     * XMLデータの設定
     *
     * @access private
     * @return void
     **/
    private function _setXml()
    {
        if ($this->summary == 'rss'){
            $this->objvew->setBaseFile('common/summary/rss.tpl');
        } elseif ($this->summary == 'atom') {
            $this->objvew->setBaseFile('common/summary/atom.tpl');
        } elseif ($this->summary == 'rdf') {
            $this->objvew->setBaseFile('common/summary/rdf.tpl');
        } else {
            return ;
        }
        $this->objvew->setMimeType('text/xml');
        $modified = empty($this->_list[0]['modified'])? 'now':$this->_list[0]['modified'];

        /* プロフィール */
        $condition = array(
            'ref_account'=> $this->member['id']
        );
        $profile = RKT_DB::getRow('act_profile', $condition, 'nickname,comment');

        $summary = array(
            'title'=>       sprintf(_("%s's Topic"), $profile['nickname']),
            'link'=>        HTTP_ACTION.'community/forum/list/'.$this->community['id'].'/',
            'modified'=>    $modified,
            'creator'=>     $profile['nickname'],
            'description'=> strip_tags($profile['comment']),
            'list'=>        $this->_list
        );

        $this->objvew->assign('summary', $summary);
    }
} // class Controller
?>
